@@ -144,6 +144,7 @@ extern int http_accept_port_number;
 
 // also the 'throttle connect headroom'
 #define EMERGENCY_THROTTLE 16
+#define THROTTLE_AT_ONCE 5
 #define HYPER_EMERGENCY_THROTTLE 6
 
 #define NET_THROTTLE_ACCEPT_HEADROOM 1.1  // 10%
@@ -270,63 +271,29 @@ check_shedding_warning()
 }
 
 TS_INLINE bool
-emergency_throttle(ink_hrtime now)
-{
-  return (bool)(emergency_throttle_time > now);
-}
-
-TS_INLINE bool
-check_net_throttle(ThrottleType t, ink_hrtime now)
+check_net_throttle(ThrottleType t)
 {
   int connections = net_connections_to_throttle(t);
 
   if (connections >= net_connections_throttle)
     return true;
 
-  if (emergency_throttle(now))
-    return true;
-
   return false;
 }
 
 TS_INLINE void
-check_throttle_warning()
+check_throttle_warning(ThrottleType type)
 {
   ink_hrtime t = Thread::get_hrtime();
   if (t - last_throttle_warning > NET_THROTTLE_MESSAGE_EVERY) {
     last_throttle_warning = t;
-    RecSignalWarning(REC_SIGNAL_SYSTEM_ERROR, "too many connections, throttling");
+    int connections       = net_connections_to_throttle(type);
+    RecSignalWarning(REC_SIGNAL_SYSTEM_ERROR,
+                     "too many connections, throttling.  connection_type=%s, current_connections=%d, net_connections_throttle=%d",
+                     type == ACCEPT ? "ACCEPT" : "CONNECT", connections, net_connections_throttle);
   }
 }
 
-//
-// Emergency throttle when we are close to exhausting file descriptors.
-// Block all accepts or connects for N seconds where N
-// is the amount into the emergency fd stack squared
-// (e.g. on the last file descriptor we have 14 * 14 = 196 seconds
-// of emergency throttle).
-//
-// Hyper Emergency throttle when we are very close to exhausting file
-// descriptors.  Close the connection immediately, the upper levels
-// will recover.
-//
-TS_INLINE bool
-check_emergency_throttle(Connection &con)
-{
-  int fd        = con.fd;
-  int emergency = fds_limit - EMERGENCY_THROTTLE;
-  if (fd > emergency) {
-    int over                = fd - emergency;
-    emergency_throttle_time = Thread::get_hrtime() + (over * over) * HRTIME_SECOND;
-    RecSignalWarning(REC_SIGNAL_SYSTEM_ERROR, "too many open file descriptors, emergency throttling");
-    int hyper_emergency = fds_limit - HYPER_EMERGENCY_THROTTLE;
-    if (fd > hyper_emergency)
-      con.close();
-    return true;
-  }
-  return false;
-}
-
 TS_INLINE int
 change_net_connections_throttle(const char *token, RecDataT data_type, RecData value, void *data)
 {
